; The M13_SM_RUN.ASM file

.586P
.MODEL FLAT, stdcall     ; Flat memory model

EXTERN NEWARRAY@4:NEAR   ; This procedure is defined in M13_externs.cpp
EXTERN OUTPUTSZ@4:NEAR   ; This procedure is defined in M13_externs.cpp
EXTERN OUTPUTINT@4:NEAR   ; This procedure is defined in M13_externs.cpp
EXTERN strlen:NEAR       ; This procedure is part of the C standard library

EXTERN global_variable:DWORD    ; Sample global variable defined in M13_externs.cpp

PUBLIC SM_RUN            ; SM_RUN is externally visible to the linker

.const
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ; Constants corresponding to the command opcodes
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        COMMAND_NOP				EQU 0 
        COMMAND_PUSH_NUMBER		EQU 1
        COMMAND_PUSH_REGISTER	EQU 2
        COMMAND_ADD				EQU 3
        COMMAND_HLT				EQU 4
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.data                   ; The data segment
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ; Table of procedure addresses invoked by the stack machine
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        call_table DWORD OFFSET nop_cmd
				   DWORD OFFSET push_cmd
				   DWORD OFFSET pushreg_cmd
                   DWORD OFFSET add_cmd
                   DWORD OFFSET hlt_cmd
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.code                    ; Code segment begins
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Procedure for running the stack machine program
; Output: EAX is the status code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
SM_RUN  PROC stack_machine_code:DWORD

		mov   eax, stack_machine_code

        ; Invoke procedures corresponding to particular opcodes
forever:
		mov     ebx, [ eax ]            ; get the opcode
		push    DWORD PTR [ eax + 4 ]   ; for each command push operand on stack
        call    call_table[ ebx * 4 ]   ; invoke command handler
        ;cmp     call_table[ ebx * 4 ], OFFSET hlt_cmd ; HALT?
        cmp     ebx, COMMAND_HLT
        je      getout                  ; Yes, we are done.
        add     eax, 8                  ; Advance to the next opcode
		jmp     forever

getout:
        mov DWORD PTR [global_variable], 456  ; Testing global variable
        mov eax, 678                      ; Procedure return value
        ret
SM_RUN  ENDP


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sample procedure handling the PUSH command
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.const
        SZ_PUSH_CMD BYTE "...push_cmd...", 13, 10, 0
.code
push_cmd PROC push_operand:DWORD
		mov   edx, push_operand  ; get the opeand from the stack
        pushad
        pushd OFFSET SZ_PUSH_CMD
        call  OUTPUTSZ@4    ; display text
        push  DWORD PTR [push_operand]
        call  OUTPUTINT@4    ; display text
        popad
        ret
push_cmd ENDP


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sample procedure handling the ADD command
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.const
        SZ_ADD_CMD BYTE "...add_cmd...", 13, 10, 0
.code
add_cmd PROC push_operand:DWORD
        pushad
        pushd OFFSET SZ_ADD_CMD
        call  OUTPUTSZ@4    ; display text
        popad
        ret
add_cmd ENDP


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sample procedure handling the NOP command
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.const
        SZ_NOP_CMD BYTE "...nop_cmd...", 13, 10, 0
.code
nop_cmd PROC push_operand:DWORD
        pushad
        pushd OFFSET SZ_NOP_CMD
        call  OUTPUTSZ@4    ; display text
        popad
        ret
nop_cmd ENDP


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sample procedure handling the HLT command
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.const
        SZ_HLT_CMD BYTE "...hlt_cmd...", 13, 10, 0
.code
hlt_cmd PROC push_operand:DWORD
        pushad
        pushd OFFSET SZ_HLT_CMD
        call  OUTPUTSZ@4    ; display text
        popad
        ret
hlt_cmd ENDP


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sample procedure handling the PUSH REG command
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
.const
        SZ_PUSHREG_CMD BYTE "...pushreg_cmd...", 13, 10, 0
.code
pushreg_cmd PROC push_operand:DWORD
        pushad
        pushd OFFSET SZ_PUSHREG_CMD
        call  OUTPUTSZ@4    ; display text
        popad
        ret
pushreg_cmd ENDP

END

